/*
** pork_util.c - utility functions
** Copyright (C) 2002-2003 Ryan McCabe <ryan@numb.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License, version 2,
** as published by the Free Software Foundation.
*/

#include <config.h>

#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>

#include <pork_missing.h>
#include <pork_util.h>

// Make something ALL CAPS.  I like it that way.
void allcaps(char *s) {

	if (s == NULL)
		return;

	for (; *s != '\0' ; s++)
		*s = toupper(*s);
}

void *xmalloc(size_t len) {
	void *ret = malloc(len);
	if (ret == NULL) {
		exit(-1);
	}

	return (ret);
}

void *xcalloc(size_t nmemb, size_t len) {
	void *ret = calloc(nmemb, len);

	if (ret == NULL) {
		exit(-1);
	}

	return (ret);
}

char *xstrdup(const char *str) {
	char *ret = strdup(str);

	if (ret == NULL) {
		exit(-1);
	}

	return (ret);
}

/*
** Copy at most n-1 characters from src to dest and nul-terminate dest.
** Returns the number of characters copied (excluding the '\0') if the whole
** string was copied, -1 otherwise.
*/

int xstrncpy(char *dest, const char *src, size_t n) {
	char *start = dest;

	if (n == 0)
		return (-1);

	while (--n > 0) {
		 if ((*dest++ = *src++) == '\0')
			return (dest - start - 1);
	}

	*dest = '\0';

	if (n > 0 && *src == '\0')
		return (dest - start);

	return (-1);
}

/*
** Append at most n-1 characters from src to dest and nul-terminate dest.
** Returns a pointer to the destination string.
*/

char *xstrncat(char *dest, const char *src, size_t n) {
	u_char *ret = dest;

	if (n == 0)
		return (dest);

	for (; *dest != '\0' ; dest++, n--)
		;
	while (--n > 0 && (*dest++ = *src++) != '\0')
		;
	*dest = '\0';

	return (ret);
}

/*
** Returns 1 if the string is blank (i.e. contains
** nothing but space characters), 0 if it's not blank.
*/

int blank_str(const char *str) {
	const char *p = str;

	while (*p != '\0') {
		if (*p != ' ')
			return (0);
		p++;
	}

	return (1);
}

/*
** Hash by Karl Nelson <kenelson@ece.ucdavis.edu>
**
** See http://mail.gnome.org/archives/gtk-devel-list/2000-February/msg00057.html
** for details.
*/

inline u_int32_t string_hash(const char *str, u_int32_t order) {
	u_int32_t hash = 0;

	while (*str != '\0')
		hash = (hash << 5) - hash + *str++;

	return (hash & ((1 << order) - 1));
}

inline int str_to_uint(const char *str, u_int32_t *val) {
	char *end;
	u_int32_t temp;

	temp = strtoul(str, &end, 0);
	if (*end != '\0')
		return (-1);

	*val = temp;
	return (0);
}

inline int str_to_int(const char *str, int *val) {
	char *end;
	int temp;

	temp = strtol(str, &end, 0);
	if (*end != '\0')
		return (-1);

	*val = temp;
	return (0);
}
